@@ -11,7 +11,7 @@ vcr::use_cassette(
         expect_s3_class(
           finbif_occurrence(
             species = "Rangifer tarandus fennicus", check_taxa = FALSE,
-            select = "record_id"
+            select = "record_id", sample = TRUE
           ),
           "finbif_occ"
         )
@@ -33,6 +33,23 @@ vcr::use_cassette(
           "finbif_occ"
         )
 
+        expect_s3_class(
+          finbif_occurrence(
+            "Pteromys volans", sample = TRUE, n = 1001, cache = FALSE
+          ),
+          "finbif_occ"
+        )
+
+        expect_s3_class(
+          finbif_occurrence("Pteromys volans", sample = TRUE, n = 1001),
+          "finbif_occ"
+        )
+
+        expect_s3_class(
+          finbif_occurrence(select =  "taxon_id", sample = TRUE, n = 3001),
+          "finbif_occ"
+        )
+
       }
 
     )
